@@ -15,6 +15,7 @@
 
 from __future__ import absolute_import
 import logging
+import six
 
 from django.conf import settings
 
@@ -34,6 +35,7 @@ CLUSTER_TEMPLATE_CREATE_ATTRS = cluster_templates.CREATION_ATTRIBUTES
 CLUSTER_CREATE_ATTRS = clusters.CREATION_ATTRIBUTES
 CERTIFICATE_CREATE_ATTRS = certificates.CREATION_ATTRIBUTES
 QUOTA_CREATION_ATTRIBUTES = quotas.CREATION_ATTRIBUTES
+CLUSTER_UPDATE_ALLOWED_PROPERTIES = set(['/node_count'])
 
 
 def _cleanup_params(attrs, create, **params):
@@ -50,7 +52,7 @@ def _cleanup_params(attrs, create, **params):
             raise exceptions.BadRequest(
                 "Key must be in %s" % ",".join(attrs))
         if key == "labels":
-            if isinstance(value, str) or isinstance(value, unicode):
+            if isinstance(value, str) or isinstance(value, six.text_type):
                 labels = {}
                 vals = value.split(",")
                 for v in vals:
@@ -80,13 +82,9 @@ def _create_patches(old, new):
     patch = []
 
     for key in new:
-        if key == 'labels':
-            # NOTE(shu-mutou): labels for cluster template and cluster
-            # are not updatable.
-            continue
         path = '/' + key
         if key in old and old[key] != new[key]:
-            if new[key] is None or new[key] is '':
+            if new[key] is None or new[key] == '':
                 patch.append({'op': 'remove', 'path': path})
             else:
                 patch.append({'op': 'replace', 'path': path,
@@ -95,22 +93,23 @@ def _create_patches(old, new):
             patch.append({'op': 'add', 'path': path, 'value': new[key]})
 
     for key in old:
-        if key == 'labels':
-            # NOTE(shu-mutou): labels for cluster template and cluster
-            # are not updatable.
-            continue
         path = '/' + key
         if key not in new:
             patch.append({'op': 'remove', 'path': path})
 
+    # convert dict value for labels into string
+    for p in patch:
+        if 'value' in p:
+            p['value'] = str(p['value'])
     return patch
 
 
 @memoized
 def magnumclient(request):
     magnum_url = ""
+    service_type = 'container-infra'
     try:
-        magnum_url = base.url_for(request, 'container-infra')
+        magnum_url = base.url_for(request, service_type)
     except exceptions.ServiceCatalogException:
         LOG.debug('No Container Infrastructure Management service is '
                   'configured.')
@@ -121,12 +120,15 @@ def magnumclient(request):
 
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
+    openstack_api_versions = getattr(settings, 'OPENSTACK_API_VERSIONS', {})
+    magnum_api_version = openstack_api_versions.get(service_type, 1.1)
 
     c = magnum_client.Client(username=request.user.username,
                              project_id=request.user.tenant_id,
                              input_auth_token=request.user.token.id,
                              magnum_url=magnum_url,
                              insecure=insecure,
+                             api_version=magnum_api_version,
                              cacert=cacert)
     return c
 
@@ -170,6 +172,13 @@ def cluster_update(request, id, **kwargs):
     old = magnumclient(request).clusters.get(id).to_dict()
     old = _cleanup_params(CLUSTER_CREATE_ATTRS, False, **old)
     patch = _create_patches(old, new)
+
+    # NOTE(flwang): Now Magnum only support updating the node count for
+    # cluster update action. So let's simplify it by only passing the
+    # /node_count dict which can avoid many potential bugs.
+    patch = [d for d in patch if d['path']
+             in CLUSTER_UPDATE_ALLOWED_PROPERTIES]
+
     return magnumclient(request).clusters.update(id, patch, rollback=rollback)
 
 
@@ -187,6 +196,26 @@ def cluster_show(request, id):
     return magnumclient(request).clusters.get(id)
 
 
+def cluster_resize(request, cluster_id, node_count,
+                   nodes_to_remove=None, nodegroup=None):
+
+    if nodes_to_remove is None:
+        nodes_to_remove = []
+
+    # Note: Magnum client does not use any return statement so result will
+    # be None unless an exception is raised.
+    return magnumclient(request).clusters.resize(
+        cluster_id, node_count,
+        nodes_to_remove=nodes_to_remove, nodegroup=nodegroup)
+
+
+def cluster_upgrade(request, cluster_uuid, cluster_template,
+                    max_batch_size=1, nodegroup=None):
+    return magnumclient(request).clusters.upgrade(
+        cluster_uuid, cluster_template,
+        max_batch_size=max_batch_size, nodegroup=None)
+
+
 def certificate_create(request, **kwargs):
     args = {}
     for (key, value) in kwargs.items():
@@ -227,11 +256,7 @@ def quotas_create(request, **kwargs):
 
 
 def quotas_update(request, project_id, resource, **kwargs):
-    new = _cleanup_params(QUOTA_CREATION_ATTRIBUTES, True, **kwargs)
-    old = magnumclient(request).quotas.get(project_id, resource).to_dict()
-    old = _cleanup_params(QUOTA_CREATION_ATTRIBUTES, False, **old)
-    patch = _create_patches(old, new)
-    return magnumclient(request).quotas.update(project_id, resource, patch)
+    return magnumclient(request).quotas.update(project_id, resource, kwargs)
 
 
 def quotas_delete(request, project_id, resource):
