@@ -12,14 +12,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from collections import defaultdict
+
+from django.conf import settings
+from django.http import HttpResponseNotFound
 from django.views import generic
 
+from magnum_ui.api import heat
 from magnum_ui.api import magnum
 
+from openstack_dashboard import api
 from openstack_dashboard.api import neutron
 from openstack_dashboard.api.rest import urls
 from openstack_dashboard.api.rest import utils as rest_utils
 
+import re
+
 
 def change_to_id(obj):
     """Change key named 'uuid' to 'id'
@@ -67,9 +75,40 @@ class ClusterTemplates(generic.View):
 
         The returned result is an object with property 'items' and each
         item under this is a Cluster Template.
+
+        If a GET query param for 'related_to' is specified, and
+        the setting for template filtering is set, then Horizon will
+        only return template groups which the given template
+        falls into, or all if none match.
         """
-        result = magnum.cluster_template_list(request)
-        return {'items': [change_to_id(n.to_dict()) for n in result]}
+        templates = magnum.cluster_template_list(request)
+
+        template_filters = getattr(
+            settings, "CLUSTER_TEMPLATE_GROUP_FILTERS", None)
+        related_to_id = request.GET.get("related_to")
+
+        if template_filters and related_to_id:
+            templates_by_id = {t.uuid: t for t in templates}
+            related_to = templates_by_id.get(related_to_id)
+
+            if related_to:
+                matched_groups = []
+                groups = defaultdict(list)
+                for group, regex in template_filters.items():
+                    pattern = re.compile(regex)
+                    if pattern.match(related_to.name):
+                        matched_groups.append(group)
+                    for template in templates:
+                        if pattern.match(template.name):
+                            groups[group].append(template)
+
+                if matched_groups:
+                    new_templates = []
+                    for group in matched_groups:
+                        new_templates += groups[group]
+                    templates = set(new_templates)
+
+        return {'items': [change_to_id(n.to_dict()) for n in templates]}
 
     @rest_utils.ajax(data_required=True)
     def delete(self, request):
@@ -116,6 +155,70 @@ class Cluster(generic.View):
             updated_cluster.to_dict())
 
 
+@urls.register
+class ClusterResize(generic.View):
+
+    url_regex = r'container_infra/clusters/(?P<cluster_id>[^/]+)/resize$'
+
+    @rest_utils.ajax()
+    def get(self, request, cluster_id):
+        """Get cluster details for resize"""
+        try:
+            cluster = magnum.cluster_show(request, cluster_id).to_dict()
+        except AttributeError as e:
+            print(e)
+            return HttpResponseNotFound()
+
+        stack = heat.stack_get(request, cluster["stack_id"])
+        search_opts = {"name": "%s-" % stack.stack_name}
+        servers = api.nova.server_list(request, search_opts=search_opts)[0]
+
+        worker_nodes = []
+        for server in servers:
+            if (server.name.startswith("%s-minion" % stack.stack_name) or
+                    server.name.startswith("%s-node" % stack.stack_name)):
+                worker_nodes.append({"name": server.name, "id": server.id})
+
+        return {"cluster": change_to_id(cluster),
+                "worker_nodes": worker_nodes}
+
+    @rest_utils.ajax(data_required=True)
+    def post(self, request, cluster_id):
+        """Resize a cluster"""
+        nodes_to_remove = request.DATA.get("nodes_to_remove", None)
+        nodegroup = request.DATA.get("nodegroup", None)
+        node_count = request.DATA.get("node_count")
+
+        # Result will be 'None' unless error is raised response will be '204'
+        try:
+            return magnum.cluster_resize(
+                request, cluster_id, node_count,
+                nodes_to_remove=nodes_to_remove,
+                nodegroup=nodegroup).to_dict()
+        except AttributeError as e:
+            # If cluster is not found magnum-client throws Attribute error
+            # catch and respond with 404
+            print(e)
+            return HttpResponseNotFound()
+
+
+@urls.register
+class ClusterUpgrade(generic.View):
+
+    url_regex = r'container_infra/clusters/(?P<cluster_id>[^/]+)/upgrade$'
+
+    @rest_utils.ajax(data_required=True)
+    def post(self, request, cluster_id):
+        """Upgrade a cluster"""
+        cluster_template = request.DATA.get("cluster_template")
+        max_batch_size = request.DATA.get("max_batch_size", 1)
+        nodegroup = request.DATA.get("nodegroup", None)
+
+        return magnum.cluster_upgrade(
+            request, cluster_id, cluster_template,
+            max_batch_size=max_batch_size, nodegroup=nodegroup).to_dict()
+
+
 @urls.register
 class Clusters(generic.View):
     """API for Magnum Clusters"""
@@ -227,9 +330,9 @@ class Quota(generic.View):
         """
         params = request.DATA
         updated = magnum.quotas_update(
-            request, project_id, resource, **params)
+            request, **params)
         return rest_utils.CreatedResponse(
-            ('/api/container_infra/quotas/%s/%s' % project_id, resource),
+            ('/api/container_infra/quotas/%s/%s' % (project_id, resource)),
             updated.to_dict())
 
     @rest_utils.ajax(data_required=True)
@@ -254,7 +357,7 @@ class Quotas(generic.View):
         item under this is a Quota.
         """
         result = magnum.quotas_list(request)
-        return {'items': [change_to_id(n.to_dict()) for n in result]}
+        return {'items': [n.to_dict() for n in result]}
 
     @rest_utils.ajax(data_required=True)
     def post(self, request):
@@ -264,8 +367,8 @@ class Quotas(generic.View):
         """
         created = magnum.quotas_create(request, **request.DATA)
         return rest_utils.CreatedResponse(
-            ('/api/container_infra/quotas/%s/%s' % created.uuid,
-             created.resource),
+            ('/api/container_infra/quotas/%s/%s' % (
+                created.project_id, created.resource)),
             created.to_dict())
 
 
