@@ -31,8 +31,11 @@
     var service = {
       createCluster: createCluster,
       updateCluster: updateCluster,
+      upgradeCluster: upgradeCluster,
       getCluster: getCluster,
       getClusters: getClusters,
+      getClusterNodes: getClusterNodes,
+      resizeCluster: resizeCluster,
       deleteCluster: deleteCluster,
       deleteClusters: deleteClusters,
       createClusterTemplate: createClusterTemplate,
@@ -73,6 +76,13 @@
         });
     }
 
+    function upgradeCluster(id, params) {
+      return apiService.post('/api/container_infra/clusters/' + id + '/upgrade', params)
+        .error(function() {
+          toastService.add('error', gettext('Unable to perform rolling upgrade.'));
+        });
+    }
+
     function getCluster(id) {
       return apiService.get('/api/container_infra/clusters/' + id)
         .error(function() {
@@ -87,6 +97,21 @@
         });
     }
 
+    function getClusterNodes(id) {
+      return apiService.get('/api/container_infra/clusters/' + id + '/resize')
+        .error(function() {
+          toastService.add('error', gettext('Unable to get cluster\'s working nodes.'));
+        });
+    }
+
+    function resizeCluster(id, params) {
+      return apiService.post('/api/container_infra/clusters/' + id + '/resize', params)
+        .error(function() {
+          var msg = gettext('Unable to resize given cluster id: %(id)s.');
+          toastService.add('error', interpolate(msg, { id: id }, true));
+        });
+    }
+
     function deleteCluster(id, suppressError) {
       var promise = apiService.delete('/api/container_infra/clusters/', [id]);
       return suppressError ? promise : promise.error(function() {
@@ -128,8 +153,9 @@
         });
     }
 
-    function getClusterTemplates() {
-      return apiService.get('/api/container_infra/cluster_templates/')
+    function getClusterTemplates(relatedTemplateId) {
+      var filterQuery = relatedTemplateId ? '?related_to=' + relatedTemplateId : '';
+      return apiService.get('/api/container_infra/cluster_templates/' + filterQuery)
         .error(function() {
           toastService.add('error', gettext('Unable to retrieve the cluster templates.'));
         });
@@ -205,8 +231,8 @@
       });
     }
 
-    function createQuota(projectId, resource, params) {
-      return apiService.post('/api/container_infra/quotas/' + projectId + '/' + resource, params)
+    function createQuota(params) {
+      return apiService.post('/api/container_infra/quotas/', params)
         .error(function() {
           toastService.add('error', gettext('Unable to create quota.'));
         });
@@ -220,7 +246,8 @@
     }
 
     function deleteQuota(projectId, resource, suppressError) {
-      var promise = apiService.delete('/api/container_infra/quotas/' + projectId + '/' + resource);
+      var promise = apiService.delete('/api/container_infra/quotas/' + projectId + '/' + resource,
+        {project_id: projectId, resource: resource});
       return suppressError ? promise : promise.error(function() {
         var msg = gettext('Unable to delete the quota with project id: %(projectId)s and ' +
           'resource: %(resource)s.');
