@@ -68,6 +68,32 @@
         "path": "/api/container_infra/clusters/",
         "error": "Unable to retrieve the clusters."
       },
+      {
+        "func": "getClusterNodes",
+        "method": "get",
+        "path": "/api/container_infra/clusters/123/resize",
+        "error": "Unable to get cluster\'s working nodes.",
+        "testInput": ["123"]
+      },
+      {
+        "func": "resizeCluster",
+        "method": "post",
+        "path": "/api/container_infra/clusters/123/resize",
+        "data": {
+          "node_count": 2,
+          "nodes_to_remove": ["456"],
+          "nodegroup": "default-worker"
+        },
+        "error": "Unable to resize given cluster id: 123.",
+        "testInput": [
+          "123",
+          {
+            "node_count": 2,
+            "nodes_to_remove": ["456"],
+            "nodegroup": "default-worker"
+          }
+        ]
+      },
       {
         "func": "deleteCluster",
         "method": "delete",
@@ -76,6 +102,25 @@
         "error": "Unable to delete the cluster with id: 1",
         "testInput": [1]
       },
+      {
+        "func": "upgradeCluster",
+        "method": "post",
+        "path": "/api/container_infra/clusters/123/upgrade",
+        "data": {
+          "cluster_template": "ABC",
+          "max_batch_size": 1,
+          "node_group": "default-worker"
+        },
+        "error": "Unable to perform rolling upgrade.",
+        "testInput": [
+          "123",
+          {
+            "cluster_template": "ABC",
+            "max_batch_size": 1,
+            "node_group": "default-worker"
+          }
+        ]
+      },
       {
         "func": "deleteClusters",
         "method": "delete",
@@ -113,6 +158,13 @@
         "path": "/api/container_infra/cluster_templates/",
         "error": "Unable to retrieve the cluster templates."
       },
+      {
+        "func": "getClusterTemplates",
+        "method": "get",
+        "path": "/api/container_infra/cluster_templates/?related_to=123",
+        "error": "Unable to retrieve the cluster templates.",
+        "testInput": [123]
+      },
       {
         "func": "deleteClusterTemplate",
         "method": "delete",
@@ -176,20 +228,18 @@
       {
         "func": "createQuota",
         "method": "post",
-        "path": "/api/container_infra/quotas/123/Cluster",
+        "path": "/api/container_infra/quotas/",
         "data": {
           "project_id": "123",
           "resource": "Cluster",
-          "hard_limit": "1"
+          "hard_limit": 1
         },
         "error": "Unable to create quota.",
         "testInput": [
-          "123",
-          "Cluster",
           {
             "project_id": "123",
             "resource": "Cluster",
-            "hard_limit": "1"
+            "hard_limit": 1
           }
         ]
       },
@@ -216,6 +266,10 @@
       {
         "func": "deleteQuota",
         "method": "delete",
+        "data": {
+          "project_id": "123",
+          "resource": "Cluster"
+        },
         "path": "/api/container_infra/quotas/123/Cluster",
         "error": "Unable to delete the quota with project id: 123 and resource: Cluster.",
         "testInput": ["123", "Cluster"]
